const BMP24 = require('./BMP24');
const font = require('./font');
const fonts = [
    font.font16x32,
    font.font12x24,
    font.font8x16
];
const p1 = 'ABCDEFGHKMNPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyz';
const p2 = '123456789';
const colors1 = [
    {red: 0, green: 0, blue: 0},
    {red: 17, green: 17, blue: 17},
    {red: 51, green: 51, blue: 51},
    {red: 85, green: 85, blue: 85},
    {red: 119, green: 119, blue: 119},
    {red: 153, green: 153, blue: 153},
    {red: 187, green: 187, blue: 187},
    {red: 221, green: 221, blue: 221},
    {red: 238, green: 238, blue: 238},
    {red: 255, green: 255, blue: 255}
];

const colors2 = [
    {red: 17, green: 17, blue: 17},
    {red: 0, green: 50, blue: 0},
    {red: 100, green: 0, blue: 0},
    {red: 150, green: 0, blue: 0},
    {red: 200, green: 0, blue: 0},
    {red: 0, green: 100, blue: 0},
    {red: 0, green: 150, blue: 0},
    {red: 0, green: 200, blue: 0},
    {red: 0, green: 0, blue: 100},
    {red: 0, green: 0, blue: 150},
    {red: 0, green: 0, blue: 200},
    {red: 30, green: 144, blue: 255},
    {red: 255, green: 215, blue: 0},
    {red: 65, green: 105, blue: 225},
    {red: 255, green: 20, blue: 147},
    {red: 255, green: 48, blue: 48},
    // {red: 0, green: 205, blue: 0},
    // {red: 0, green: 139, blue: 0},
    // {red: 51, green: 51, blue: 51}
    // {red: 17, green: 17, blue: 17},
    // {red: 17, green: 17, blue: 17},
    // {red: 17, green: 17, blue: 17},
    // {red: 17, green: 17, blue: 17},
];

function rand(min, max) {
    return Math.random() * (max - min + 1) + min | 0; // 特殊的技巧，|0可以强制转换为整数
}

function randRGB(colorful) {
    if (!colorful) {
        return colors1[rand(0, 4)];
    }
    return colors2[rand(0, 15)];
    // var seed = rand(0, 0xffffff);
    // return {
    //     blue: (seed >> 8) & 0xFF,
    //     green: (seed >> 16) & 0xFF,
    //     red: (seed >> 32) & 0xFF
    // };
}

module.exports = {
    /** 制造验证码图片
     * @param text {String} 验证码文本
     * @param colorful {Boolean} 是否彩色
     * @param level {Number} 难度等级， 从底到高 1 2 3
     */
    make: function(text, colorful, level) {
        level = (level % 4) || 1;
        
        const imgLen = text.length * 20 + 8;
        const imgHeight = 46;
        const img = new BMP24(imgLen, imgHeight);
        img.fillRect(0, 0, imgLen, imgHeight, colors1[rand(9, 9)]);

        // 加干扰线
        img.drawLine(rand(0, 100), rand(0, 40), rand(0, 100), rand(0, 40), randRGB(colorful));
        img.drawCircle(rand(0, 100), rand(0, 40), rand(10, 40), randRGB(colorful));

        if (level > 1) {
            img.drawLine(rand(0, 60), rand(0, 30), rand(0, 60), rand(0, 30), randRGB(colorful));
            img.drawLine(rand(0, 100), rand(0, 40), rand(0, 100), rand(0, 40), randRGB(colorful));
            img.drawCircle(rand(0, 30), rand(0, 15), rand(10, 20), randRGB(colorful));
        }
        if (level > 2) {
            img.drawLine(rand(0, 100), rand(0, 40), rand(0, 100), rand(0, 40), randRGB(colorful));
            img.drawCircle(rand(0, 60), rand(0, 30), rand(20, 40), randRGB(colorful));
            img.drawRect(rand(0, 100), rand(0, 100), rand(0, 20), rand(0, 20), randRGB(colorful));
        }
        // 画曲线
        const w = imgLen / 2;
        const h = imgHeight;
        let y1 = rand(-5, 5); // Y 轴位置调整
        let w2 = rand(10, 15); // 数值越小频率越高
        let h3 = rand(4, 6); // 数值越小幅度越大

        let max = [1, 2, 4, 5][level];
        let bl = rand(1, max);

        for (let i = -w; i < w; i += 0.1) {
            let y = Math.floor(h / h3 * Math.sin(i / w2) + h / 2 + y1);
            let x = Math.floor(i + w);
            for (let j = 0; j < bl; j++) {
                img.drawPoint(x, y + j, randRGB(colorful));
            }
        }
        if (level > 2) {
            y1 = rand(-5, 5); // Y 轴位置调整
            w2 = rand(10, 15); // 数值越小频率越高
            h3 = rand(4, 6); // 数值越小幅度越大
            bl = rand(1, max);
            for (let i = -w; i < w; i += 0.3) {
                let y = Math.floor(h / h3 * Math.tan(i / w2) + h / 2 + y1);
                let x = Math.floor(i + w);
                for (let j = 0; j < bl; j++) {
                    img.drawPoint(x, y + j, randRGB(colorful));
                }
            }
        }
        let x = 8, y = 8;
        max = [0, 0, 1, 2][level];
        for (const ch of text) {
            const f = fonts[rand(0, max)];
            y = 8 + rand(-10, 10);
            img.drawChar(ch, x, y, f, randRGB(colorful));
            x += f.w + rand(2, 8);
        }
        return img;
    },
    // 生成随机文本
    text: function(len, numOnly) {
        let str = '';
        var p = numOnly ? p2 : p1;
        for (let i = 0; i < len; i++) {
            str += p.charAt(Math.random() * p.length | 0);
        }
        return str;
    }
}
